# HLIN -- Dispersion correct and linearize spectrum, placing input in Nodisp/
# By Doug Mink, SAO Telescope Date Center
# August 10, 2005

procedure hlin (images)

string	images = ""	{prompt="Input multispec image(s) to correct"}
string	calfile=""	{prompt="Calibration file for dispersion correction"}
bool	findshift = yes	{prompt="Compute spectrum pixel shift (yes or no)"}
int     shiftline=1     {prompt="Line to shift in position file"}
file    emlines="hectosky1.dat" {prompt="File of emission line positions"}
file    linedir="rvsao$lib/"    {prompt="Directory for emission line positions"}
string  aprange="50-100"	{prompt="Range of apertures over which to average pixel shift"}
bool	median=yes	{prompt="Shift is median (yes) or mean (no) shift"}
bool	saveshift=yes	{prompt="Save shift in header (yes or no)"}
bool	linearize = yes	{prompt="Linearize spectra (yes or no)"}
real	wl1=3700.0	{prompt="Starting wavelength for output spectra (A)"}
real	wl2=9150.0	{prompt="Ending wavelength for output spectra (A)"}
real	dwl=1.21	{prompt="Delta wavelength per pixel for output spectra (A)"}
int	npix = INDEF	{prompt="Number of pixels per output spectra"}
bool	deltemp = yes	{prompt="Delete temporary files (y or n)"}
bool	saveold = yes	{prompt="Save original files in Nodisp/ (y or n)"}
bool	saveunshift = yes	{prompt="Save files without shift in Noshift/ (y or n)"}
bool	savedisp = yes	{prompt="Save nonlinear dispersed files in Nolin/ (y or n)"}
bool	verbose = no	{prompt="Log progress (y or n)"}
bool	debug = no	{prompt="Too much info from sumspec (y or n)"}

begin
	bool vb, db
	file imin
	string ims, tchar, templist, arg

	if (!deftask ("emsao")) {
	    rvsao (motd=no)
	    }
	if (!deftask ("getcol")) {
	    wcstools (motd=no)
	    }

	ims = images
	vb = verbose
	db = debug
	if (db) {
	    vb = yes
	    deltemp = no
	    }

	if (ims == "") {
	    print ("HLIN:  Dispersion-correct and linearize spectra")
	    print ("Usage: hlin <input images>")
	    return
	    }

#  Make temporary list of input files
	tchar = substr (ims,1,1)
        if (tchar == '@') {
	    templist = substr (ims,2,100)
	    }
	else {
	    templist = mktemp ("linlist")
	    files (ims, >templist)
	    }
#	print (templist)

	hdisp1.calfile = calfile
	hdisp1.saveold = savedisp
	hdisp1.verbose = vb
	hdisp1.debug = db

	if (findshift) {
	    hpixshift1.shiftline = shiftline
	    hpixshift1.emlines = emlines
	    hpixshift1.linedir = linedir
	    hpixshift1.aprange = aprange
	    hpixshift1.median = median
	    hpixshift1.saveshift = saveshift
	    hpixshift1.saveold = saveunshift
	    hpixshift1.deltemp = deltemp
	    hpixshift1.verbose = vb
	    hpixshift1.debug = db
	    }

	if (linearize) {
	    hlin1.wl1 = wl1
	    hlin1.wl2 = wl2
	    hlin1.dwl = dwl
	    hlin1.npix = npix
	    hlin1.saveold = saveold
	    hlin1.verbose = vb
	    hlin1.debug = db
            }

	list = templist
	while (fscan (list,imin) != EOF) {

# Add dispersion functions to headers of input files
	    hdisp1 (imin)

# Compute pixel shifts of individual spectra in input files
	    if (findshift) {
		hpixshift1 (imin)
		}

# Linearize input files in wavelength
	    if (linearize) {
		hlin1 (imin)
		}
            }

# Delete list file if temporary and requested
	if (deltemp && tchar != '@') {
	    delete (templist)
	    }

end
# Jun 10 2004	New task
# Sep  2 2004	Run on multiple files via list argument or file
# Sep  2 2004	Add keyword HLIN and do not linearize image when it already exists
# Oct 13 2004	Drop extra closing brace
# Oct 18 2004	If already processed, print HLIN value

# Jan 12 2005	Read number of apertures from image header
# Mar  7 2005	Separate options to save original and dispersed files
# Mar 21 2005	Do not change deltemp if reading from file list
# May 11 2005	Set deltemp=no and verbose=yes if debug=yes
# May 16 2005	Separate image processing into a separate task
# May 19 2005	Separate dispersion and linearization into HDISP1 and HLIN1
# May 20 2005	Add HPIXSHIFT to compute pixel shift from nonlinear spectra
# May 27 2005	Add emlines and linedir parameters for HPIXSHIFT
# Jun  3 2005	Add findshift to make shift computation optional
# Jun  8 2005	Use HPIXSHIFT1 and add to image loop
# Jun 27 2005	Add median and saveunshift options for HPIXSHIFT1
# Aug 10 2005	Load wcstools and rvsao only if they are not already loaded
