# HPIXSHIFT1 - Use emission line to find multispec spectra shift
# June 30, 2005
# By Doug Mink, SAO Telescope Data Center

procedure hpixshift1 (image)

string	image=""	{prompt="Image file for which to compute shifts"}

int	shiftline=1	{prompt="Line to shift in position file"}
file	emlines="hectosky.dat"	{prompt="File of emission line positions"}
file	linedir="rvsao$lib/"	{prompt="Directory for emission line positions"}
string	aprange="1-300" {prompt="Apertures over which to average pixel shift"}
bool	median=yes	{prompt="Set median line shift if yes, mean if no)"}
bool	saveshift=yes	{prompt="Save shift in header (yes or no)"}
bool	saveold = yes	{prompt="Save original files in Noshift/ (y or n)"}
bool	deltemp=yes	{prompt="Delete intermediate image files (y or n)"}
bool	verbose=yes	{prompt="Print shifts (y or n)"}
bool	debug=no	{prompt="Print debugging information (y or n)"}

struct *list1

begin

file	tempemsao, linefile
file	imin, newfile
string	history, nline
string	arg, cshift, lname
real	shift, wl, wl1, wl2, dwl
bool	vb, db, save

	vb = verbose
	db = debug
	save = saveold
	imin = image

	if (imin == "") {
	    print ("HPIXSHIFT1: Find pixel shift in multispec night sky spectra")
	    print ("           Run this task on a file which has been dispersion-corrected,")
	    print ("           but not linearized, where pixels are not linear in wavelength.")
	    print ("Usage: hpixshift1 <input image>")
	    return
	    }

# Return if this file already has shift set in header
	imgetp.default = "0"
	imgetp (imin, "HPIXSHIF")
	if (imgetp.value != "0") {
	    print ("HPIXSHIFT1 has already been run.")
	    print ("HPIXSHIFT: ", imgetp.value)
	    return
	    }

#  Figure out which line to check
	if (shiftline < 1) {
	    print ("HPIXSHIFT1: Wrong line number ",shiftline)
	    return
	    }

# Find line name in line list file
	linefile = linedir // emlines
	list1 = linefile
	for (i = 0; i < shiftline; i=i+1) {
	    if (fscan (list1, wl, wl1, wl2, dwl, lname) == EOF) {
		print ("HPIXSHIFT1: Fewer than ",shiftline," lines available")
		return
		}
#	    print (i,wl,lname)
	    }
	if (vb) {
	    print ("HPIXSHIFT1: Shift measured from ", lname)
	    }
	nline = shiftline + 7

#  Set up emsao parameters
	emsao.specnum = aprange
	emsao.specband = 1
	emsao.skynum = ""
	emsao.skyband = 0
	emsao.specdir = ""
	emsao.linefit = yes
	emsao.fixbad = no
	emsao.badlines = "badlines.dat"
	emsao.renormalize = no
	emsao.st_lambda = INDEF
	emsao.end_lambda = INDEF
	emsao.nsmooth = 0
	emsao.vel_init = "guess"
	emsao.czguess = 0.
	emsao.cortemp = ""
	emsao.wspan = 10.
	emsao.linesig = 1.
	emsao.emsearch = ""
	emsao.emlines = emlines
	emsao.linedir = linedir
	emsao.npfit = 2
	emsao.nlcont = 1
	emsao.esmooth = 2
	emsao.emcombine = ""
	emsao.mincont = 0.
	emsao.lwmin = 0.4
	emsao.lwmax = 1.7
	emsao.lsmin = 1.
	emsao.sigline = 0.
	emsao.disperr = 0.01
	emsao.vel_corr = "none"
	emsao.report_mode = 11
	emsao.archive = no
	emsao.save_vel = no
	emsao.verbose = yes
#	emsao.logfiles = "STDOUT"
	emsao.device = "stdgraph"
	emsao.hardcopy = no
	if (vb && db) {
	    emsao.displot = yes
	    }
	else {
	    emsao.displot = no
	    }
	emsao.plotter = "stdplot"
	emsao.dispmode = 2
	emsao.vel_plot = "emission"
	emsao.curmode = no
	emsao.dispem = yes
	emsao.dispabs = no
	emsao.ablines = "ablines.dat"
	emsao.obj_plot = no
	emsao.contsub_plot = no
	emsao.debug = no
	emsao.nsum = 1
	emsao.cursor = ""
#       Compute shift of selected line using EMSAO
        tempemsao = mktemp ("shiftemsao")
        if (db) {
            emsao.logfiles = "STDOUT," // tempemsao
            }
        else {
            emsao.logfiles = tempemsao
            }
        if (db) {
            dpar emsao
            }
#       print ("HPIXSHIFT: emsao -> ",tempemsao)
        emsao (imin)

# Compute mean pixel shift for selected emission line and read it back
        if (median) {
            arg = "-ep " // tempemsao // " " // nline
            }
        else {
            arg = "-mp " // tempemsao // " " // nline
            }
        cshift = ""
        getcol (arg) | scan (cshift)
	if (deltemp) {
	    delete (tempemsao)
	    }

# If no pixel shift can be read, return
	if (cshift == "") {
	    print ("HPIXSHIFT1: No shift returned from EMSAO")
	    return
	    }
	shift = real (cshift)

# Move original file to Noshift directory, if requested
	if (save) {
	    if (!access ("Noshift/")) {
		mkdir ("Noshift")
		}
	    newfile = "Noshift/" // imin
	    if (access (newfile)) {
		imdelete (newfile)
		}
	    copy (imin, "Noshift")
	    if (vb) {
		print ("HLIN1: Nonlinear ", imin, " => Noshift/")
		}
	    }
	else {
	    delete (imin)
	    }


# Add pixel shift to header
	gettime ()
	history = gettime.date // " " // gettime.time // " "
	history = history // lname // ": " // shift // " pix shift, aps " // aprange
	if (median) {
	    history = history // " median"
	    }
	else {
	    history = history // " mean"
	    }
	if (vb) {
	    print ("HPIXSHIFT1: ", imin, ": ", history)
	    }
	hedit (imin, "HPIXSHIF", history, add=yes, verify=no, show=db)
	cshift = shift
	hedit (imin, "PIXSHIFT", cshift, add=yes, verify=no, show=db)

	return

end

# May 20 2005	Task to compute night sky spectrum shifts
# May 26 2005	Identify line from which to compute shift by its number
# May 27 2005	Add emlines = file of line positions, linedir = directory
# Jun  1 2005	Add apertures to HPIXSHIF keyword value
# Jun  6 2005	Add line name to HPIXSHIF keyword value
# Jun 23 2005	Check for HPIXSHIF keyword before doing anything else
# Jun 24 2005	Add mean or median to history
# Jun 24 2005	Return with error message if no EMSAO output file is found
# Jun 27 2005	Add option to save image without shift in header
# Jun 30 2005	Load wcstools and rvsao in hectospec.cl
# Jun 30 2005	Set emsao.verbose=yes always; plot only if debug and verbose
