# HPREAMP - Run Brian McCleod's preamplifier fix on a set of images
# January 3, 2005
# By Doug Mink, SAO Telescope Data Center

procedure hpreamp (images)

string	images=""	{prompt="Images to merge"}

bool	deltemp=yes	{prompt="Delete intermediate image files (y or n)"}
bool	saveold=yes	{prompt="Save unprocessed image files (y or n)"}
bool	verbose=yes	{prompt="Print process progress (y or n)"}

begin

file	imroot, temp1, imin, templist, imout
string	arg1, imx, ims, imsec, history, tchar
bool	dellist, vb

	vb = verbose
	imx = ".fits"
	ims = image

	if (ims == "") {
	    print ("HPREAMP: Run image processing tasks one amp at a time")
	    print ("Usage: hproc <input images with 4 extensions>")
	    return
	    }

# Make directory in which to save raw multi-extension files
	if (saveold) {
	    if (!access ("Raw"))
		mkdir ("Raw")
	    }

#  Make temporary list of input files
	tchar = substr (ims,1,1)
	if (tchar == '@') {
	    templist = substr (ims,2,100)
	    deltemp = no
	    }
	else {
	    templist = mktemp ("proclist")
	    files (ims, >templist)
	    }
	list = templist
        while (fscan (list,imin) != EOF) {

# Do not process images without extensions
	    imgetp.default = "-1"
	    imsec = imin // "[0]"
	    imgetp (imsec, "i_naxis")
	    if (imgetp.value != "0") {
		print ("HPREAMP: ", imin, " is not a multi-extension image")
		print ("NAXIS = ", imgetp.value)
		next
		}

# Do not process image twice
	    imgetp.default = "0"
	    imsec = imin // "[1]"
	    imgetp (imsec, "PREAMPFX")
	    if (imgetp.value != "0") {
		if (vb) {
		    print ("HPREAMP: ", imin, " has already been processed")
		    print ("HPREAMP = ", imgetp.value)
		    }
		next
		}

# Save unchanged image
	    if (saveold) {
		imout = "Raw/" // imin
		copy (imin, imout)
		}

# Find root name of image
	    imroot = imin
	    smatch (imroot, imx)
	    if (smatch.found) {
		imroot = substr (imroot,1,smatch.start-1)
		}
	    else {
		imroot = imageout
		imin = imin // imx
		}
#	    if (vb) {
#		print ("HPREAMP: image root is ",imroot)
#		}

# Fix preamp output using Brian McLeod's task
	    temp1 = mktemp (imroot) // "p.fits"
	    if (verbose)
		print ("HPREAMP: Making preamp correction for ",imin)
	    preampfix (imin, temp1)
	    if (verbose)
		print ""

# Make history line including time for preamp correction
            gettime ()
	    history = gettime.date // " " // gettime.time // " preamplifier correction made"
	    imsec = temp1 // "[1]"
	    hedit (imsec, "PREAMPFX", history, add+, verify-)
	    imsec = temp1 // "[2]"
	    hedit (imsec, "PREAMPFX", history, add+, verify-)
	    imsec = temp1 // "[3]"
	    hedit (imsec, "PREAMPFX", history, add+, verify-)
	    imsec = temp1 // "[4]"
	    hedit (imsec, "PREAMPFX", history, add+, verify-)

#  Rename result to same as original file
	    delete (imin)
	    rename (temp1, imin)
	    if (verbose)
		print ("HPREAMP: ",temp1," -> ", imin)
	    }

	if (deltemp)
	    delete (templist)
end

# Jan 03 2005	New program to only run preampfix
