# HREDLEAK1 -- Fit and remove 'red leak' continuum from spectra in one file
# By Bill Wyatt; modified by Doug Mink
# June 9, 2005
#
# Requires 'hectospec' package for gettime task
# Requires 'images.imfit' package for fit1d task
#

procedure hredleak1 (image)

string	image = "" {prompt="input file"}
real	lambda = 8520.0
bool	dounused = no    # if 'yes' correct even unused bands
string	fitfunc = "spline3"
int	order = 1
real	low_reject = 3.0
real	high_reject = 2.0
int	niterate = 20
bool	deltemp = no	{prompt="Delete temporary files (y or n)"}
bool	saveold = yes	{prompt="Save unprocessed files to Notran/ (y or n)"}
bool	verbose = yes	{prompt="Log progress (y or n)"}
bool	debug = no	{prompt="Log each aperture (y or n)"}

begin

	bool db, vb
	file imin, imout
	file newfile
	file fitfile
	string section,  sect1,  sect2
	int    deltapix
	int nap
	int lroot
	int    pixbeg,   pixend
	int    naxis
	int    naxis2
	string keyval
	int    crpix1
	real   cdelt1,   crval1
	real   lomean
	real   himean

	string history,  hdata

	db = debug
	vb = verbose

# Make sure filename ends in .ms.fits
	imin = image
	smatch (imin, ".ms")
	if (smatch.found) {
	    lroot = smatch.start - 1
	    imout = substr (imin, 1, lroot) // ".red.ms.fits"
	    smatch (imin, ".fits")
	    if (!smatch.found) {
		imin = imin // ".fits"
		}
	    }
	else {
	    smatch (imin, ".fits")
	    if (smatch.found) {
		lroot = smatch.start - 1
		imin = substr (imin, 1, lroot) // ".ms.fits"
		imout = substr (imin, 1, lroot) // ".red.ms.fits"
		}
	    else {
		imout = imin // ".red.ms.fits"
		imin = imin // ".ms.fits"
		}
	    }

	if (!access (imin)) {
	    print ("HREDLEAK1: ", imin," does not exist")
	    return
	    }

# Make sure redleak not done twice
	imgetp.default = "0"
	imgetp (imin, "REDLEAK")
	if (imgetp.value != "0") {
	    printf("REDLEAK already done on %s\n", imin)
	    return
	    }

	if (access (imout)) {
	    delete (imout)
	    }

	naxis2  = 1
	hdata   = "lambda " // str(lambda) // " fit " // fitfunc
	hdata = hdata // " O( " // str(order) // ")"

# get key values from header
	imgetp.verbose = no
	imgetp.default = "0"
	imgetp (imin, "i_naxis")
	if (imgetp.value == "0") {
	    printf("redleakcorr: file %s: no NAXIS?\n", imin)
	    error(1,"exiting...")
	    }
	naxis = int (imgetp.value)

	imgetp.default = "0"
	imgetp (imin, "i_naxis1")
	if (imgetp.value == "0") {
	    printf("redleakcorr: file %s: no NAXIS1?\n", imin)
	    error(1,"exiting...")
	    }
	pixend = int (imgetp.value)

	imgetp.default = "0"
	imgetp (imin, "i_naxis2")
	if (imgetp.value == "0") {
	    nap = 1
	    }
	if (strlen (imgetp.value) > 0) 
	    nap = int (imgetp.value)
	else
	    nap = 1

	imgetp.default = "0"
	imgetp (imin, "CRPIX1")
	if (imgetp.value == "0") {
	    printf("HREDLEAK1: file %s: no CRPIX1\n", imin)
	    error(1,"exiting...")
	    }
	crpix1 = int(imgetp.value)

	imgetp.default = "0"
	imgetp (imin, "CDELT1")
	if (imgetp.value == "0") {
	    imgetp (imin, "CD1_1")
	    if (imgetp.value == "0") {
		printf("HREDLEAK1: file %s: no CDELT1 or CD1_1\n", imin)
		error(1,"exiting...")
		}
	    }
	cdelt1 = real(imgetp.value)

	imgetp.default = "0"
	imgetp (imin, "CRVAL1")
	if (imgetp.value == "0") {
	    printf("HREDLEAK1: file %s: no CRVAL1\n", imin)
	    error(1,"exiting...")
	    }
	crval1 = real(imgetp.value)

	pixbeg = int((lambda - crval1)/cdelt1 + crpix1 - 1)

	if (vb)
	   printf("HREDLEAK1: lam= %.1f, sec [%d:%d], crpix= %d, crval= %.1f cdelt= %.3f\n",
	     0, lambda, pixbeg, pixend, crpix1, crval1, cdelt1)

# most of output is a copy of the input
	imcopy (imin, imout, verbose=no)

	fitfile  = osfn("tmp$") // mktemp("drdlk_") // ".fits"
	delete (fitfile,  ver-, >& "dev$null")

# Loop here for multiple apertures
	for (i = 1; i <= nap; i = i+1) {
	
	    imgetp.default = "0"
	    imgetp (imin, "APID" // str(i))
	    keyval = imgetp.value
	    if (! dounused && strlen(keyval) > 0 && substr(keyval,2,7) == "unused") {
		if (vb) {
	  	    printf("HRDLEAK1(%3d): %60s unused\n", i, " ")
	  	    next
	  	    }
	  	}
	    section = "[" // str(pixbeg) // ":" // str(pixend) // "," // str(i) // "]"

	    fit1d (imin // section,  fitfile,  "fit",  axis=1,
	       inter-,                 sample="*",              naver=1,
	       function=fitfunc,       order=order,             grow=0.,
	       low_reject=low_reject,  high_reject=high_reject, niterate=niterate)
	
# If the continuum trends downwards there is no (significant) red leak and
# the subtraction should not be performed

	    deltapix = (pixend - pixbeg) / 10
	    if(deltapix < 3) {
		deltapix = min(3,pixend-pixbeg+1)
		}

# get mean of first 10% of fit
	    sect1 = "["//str(pixbeg)//":"//str(pixbeg+deltapix-1) // "," //str(i)// "]"
	    imstatistics(fitfile // sect1, 
	          format=no, fields="mean", nclip=0, lower=INDEF, upper=INDEF) |
	    scanf("%f", lomean)
	
# get mean of last 10% of fit
	    sect2 = "["//str(pixend-deltapix+1)//":"//str(pixend)//","//str(i)// "]"
	    imstatistics(fitfile // sect2,
	          format=no, fields="mean", nclip=0, lower=INDEF, upper=INDEF) |
	    scanf("%f", himean)

	    if (db) {
		printf("rdlc(%3d): Fit mean%-15s= %6.1f  mean%-15s= %6.1f", 
	            i, sect1, lomean, sect2, himean)
	  	}

	    if (himean > lomean) {

		gettime(usesec=yes,verbose=no)

	  # subtract off the fit
		imexpr("a - b + lov(b)",  imout // section, 
		       imin // section,  fitfile // section,  verbose-)
		if (db) printf(" corr")
		}
	    if (db) printf("\n")

# End aperture loop
	    }

	delete (fitfile,  ver-, >& "dev$null")

# Move original file to Nored directory
	if (saveold) {
	    if (!access ("Nored/")) {
		mkdir ("Nored")
		}
	    newfile = "Nored/" // imin
	    if (access (newfile)) {
		imdelete (newfile)
		}
	    move (imin, "Nored")
	    }
	else {
	    delete (imin)
	    }

# Rename result file to original file name
	imrename (imout, imin)

	history = gettime.date// " " // gettime.time // " " // hdata
	hedit (imin, "REDLEAK", history, add+, update+, show-, verify-)
	
end
